/*
 * Unplayer
 * Copyright (C) 2015-2019 Alexey Rochev <equeim@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.2
import Sailfish.Silica 1.0

import harbour.unplayer 0.1 as Unplayer

Page {
    id: page

    SearchPanel {
        id: searchPanel
    }

    RemorsePopup {
        id: remorsePopup
    }

    SelectionPanel {
        id: selectionPanel
        selectionText: qsTranslate("unplayer", "%n file(s) selected", String(), directoryTracksProxyModel.selectedIndexesCount)

        PushUpMenu {
            MenuItem {
                enabled: directoryTracksProxyModel.hasSelection
                text: qsTranslate("unplayer", "Replace queue")
                onClicked: {
                    Unplayer.Player.queue.addTracksFromUrls(directoryTracksProxyModel.getSelectedTracks(), true)
                    selectionPanel.showPanel = false
                }
            }

            MenuItem {
                enabled: directoryTracksProxyModel.hasSelection
                text: qsTranslate("unplayer", "Add to queue")
                onClicked: {
                    Unplayer.Player.queue.addTracksFromUrls(directoryTracksProxyModel.getSelectedTracks())
                    selectionPanel.showPanel = false
                }
            }

            MenuItem {
                enabled: directoryTracksProxyModel.hasSelection
                text: qsTranslate("unplayer", "Add to playlist")
                onClicked: pageStack.push(addToPlaylistPage)

                Component {
                    id: addToPlaylistPage

                    AddToPlaylistPage {
                        tracks: directoryTracksProxyModel.getSelectedTracks()
                        Component.onDestruction: {
                            if (added) {
                                selectionPanel.showPanel = false
                            }
                        }
                    }
                }
            }

            MenuItem {
                enabled: directoryTracksProxyModel.hasSelection
                text: qsTranslate("unplayer", "Edit tags")
                onClicked: pageStack.push("TagEditDialog.qml", {files: directoryTracksProxyModel.getSelectedTracks()})
            }

            MenuItem {
                enabled: directoryTracksProxyModel.hasSelection
                text: qsTranslate("unplayer", "Remove")
                onClicked: remorsePopup.execute(qsTranslate("unplayer", "Removing %n files", String(), directoryTracksProxyModel.selectedIndexesCount), function() {
                    directoryTracksModel.removeTracks(directoryTracksProxyModel.selectedSourceIndexes)
                })
            }
        }
    }

    AsyncLoadingListView {
        id: listView

        property var savedPositions: []
        property bool goingUp

        anchors {
            fill: parent
            bottomMargin: selectionPanel.visible ? selectionPanel.visibleSize : 0
            topMargin: searchPanel.visibleSize
        }
        clip: true

        page: page
        emptyText: qsTranslate("unplayer", "No files")

        header: Column {
            property int oldHeight

            property int extraHeight: parentDirectoryItem.height

            width: listView.width

            onHeightChanged: {
                if (height > oldHeight) {
                    listView.contentY -= (height - oldHeight)
                }
                oldHeight = height
            }

            Component.onCompleted: oldHeight = height

            PageHeader {
                title: qsTranslate("unplayer", "Directories")
                description: directoryTracksModel.directory
            }

            ParentDirectoryItem {
                id: parentDirectoryItem

                visible: !directoryTracksModel.loading && directoryTracksModel.directory !== "/"
                onClicked: {
                    if (!selectionPanel.showPanel) {
                        searchPanel.open = false
                        listView.goingUp = true
                        directoryTracksModel.directory = directoryTracksModel.parentDirectory
                    }
                }
            }
        }
        delegate: ListItem {
            id: fileDelegate

            function remove() {
                remorseAction(qsTranslate("unplayer", "Remove"), function() {
                                  directoryTracksModel.removeTrack(directoryTracksProxyModel.sourceIndex(model.index))
                              })
            }

            property bool current: model.filePath === Unplayer.Player.queue.currentFilePath

            menu: Component {
                ContextMenu {
                    MenuItem {
                        visible: model.isDirectory
                        text: qsTranslate("unplayer", "Set as default directory")
                        onClicked: Unplayer.Settings.defaultDirectory = model.filePath
                    }

                    MenuItem {
                        visible: !model.isDirectory
                        text: qsTranslate("unplayer", "Track information")
                        onClicked: pageStack.push("TrackInfoPage.qml", { filePath: model.filePath })
                    }

                    MenuItem {
                        visible: !model.isDirectory
                        text: qsTranslate("unplayer", "Add to queue")
                        onClicked: Unplayer.Player.queue.addTrackFromUrl(model.filePath)
                    }

                    MenuItem {
                        visible: !model.isDirectory
                        text: qsTranslate("unplayer", "Add to playlist")
                        onClicked: pageStack.push("AddToPlaylistPage.qml", { tracks: [model.filePath] })
                    }

                    MenuItem {
                        visible: !model.isDirectory
                        text: qsTranslate("unplayer", "Edit tags")
                        onClicked: pageStack.push("TagEditDialog.qml", {files: [model.filePath]})
                    }

                    MenuItem {
                        text: qsTranslate("unplayer", "Remove")
                        onClicked: fileDelegate.remove()
                    }
                }
            }
            showMenuOnPressAndHold: !selectionPanel.showPanel

            onClicked: {
                if (selectionPanel.showPanel) {
                    if (!model.isDirectory) {
                        directoryTracksProxyModel.select(model.index)
                    }
                } else {
                    if (model.isDirectory) {
                        searchPanel.open = false
                        listView.savedPositions.push(listView.contentY + listView.headerItem.height)
                        listView.goingUp = false
                        directoryTracksModel.directory = model.filePath
                    } else {
                        if (current) {
                            if (!Unplayer.Player.playing) {
                                Unplayer.Player.play()
                            }
                        } else {
                            if (model.isPlaylist) {
                                Unplayer.Player.queue.addTracksFromUrls(directoryTracksModel.getTrack(directoryTracksProxyModel.sourceIndex(model.index)),
                                                                              true)
                            } else {
                                Unplayer.Player.queue.addTracksFromUrls(directoryTracksModel.getTracks(directoryTracksProxyModel.sourceIndexes, false),
                                                                              true,
                                                                              model.index - directoryTracksProxyModel.directoriesCount)
                            }
                        }
                    }
                }
            }

            Binding {
                target: fileDelegate
                property: "highlighted"
                value: down || menuOpen || directoryTracksProxyModel.isSelected(model.index)
            }

            Connections {
                target: directoryTracksProxyModel
                onSelectionChanged: fileDelegate.highlighted = directoryTracksProxyModel.isSelected(model.index)
            }

            Image {
                id: icon

                anchors {
                    left: parent.left
                    leftMargin: Theme.horizontalPageMargin
                    verticalCenter: parent.verticalCenter
                }
                asynchronous: true
                source: {
                    var iconSource = model.isDirectory ? "image://theme/icon-m-folder"
                                                       : "image://theme/icon-m-music"
                    if (highlighted || current) {
                        iconSource += "?" + Theme.highlightColor
                    }
                    return iconSource
                }
            }

            Label {
                anchors {
                    left: icon.right
                    leftMargin: Theme.paddingMedium
                    right: parent.right
                    rightMargin: Theme.horizontalPageMargin
                    verticalCenter: parent.verticalCenter
                }
                text: Theme.highlightText(model.fileName, searchPanel.searchText, Theme.highlightColor)
                color: highlighted || current ? Theme.highlightColor : Theme.primaryColor
                truncationMode: TruncationMode.Fade
                textFormat: Text.StyledText
            }
        }
        model: Unplayer.DirectoryTracksProxyModel {
            id: directoryTracksProxyModel
            sourceModel: Unplayer.DirectoryTracksModel {
                id: directoryTracksModel

                onLoaded: {
                    if (listView.goingUp) {
                        if (listView.savedPositions.length) {
                            listView.contentY = (listView.savedPositions.pop() - listView.headerItem.height)
                        } else {
                            listView.positionViewAtBeginning()
                        }
                    } else {
                        listView.positionViewAtBeginning()
                    }
                }
            }
        }

        PullDownMenu {
            id: pullDownMenu

            property string directory

            function goBegin(directory) {
                listView.savedPositions = []
                listView.goingUp = false
                pullDownMenu.directory = directory
                pullDownMenu.activeChanged.connect(goEnd)
            }

            function goEnd() {
                directoryTracksModel.directory = directory
                pullDownMenu.activeChanged.disconnect(goEnd)
            }

            MenuItem {
                text: qsTranslate("unplayer", "Set as default directory")
                onClicked: Unplayer.Settings.defaultDirectory = directoryTracksModel.directory
            }

            MenuItem {
                text: qsTranslate("unplayer", "Default directory")
                onClicked: pullDownMenu.goBegin(Unplayer.Settings.defaultDirectory)
            }

            MenuItem {
                text: qsTranslate("unplayer", "SD card")
                onClicked: pullDownMenu.goBegin(Unplayer.Utils.sdcardPath)
            }

            MenuItem {
                text: qsTranslate("unplayer", "Home directory")
                onClicked: pullDownMenu.goBegin(Unplayer.Utils.homeDirectory)
            }

            SelectionMenuItem {
                enabled: directoryTracksProxyModel.tracksCount || (searchPanel.searchText && directoryTracksProxyModel.tracksCount)
                text: qsTranslate("unplayer", "Select files")
            }

            SearchMenuItem { }
        }
    }
}
